@@ -4,18 +4,17 @@ from ..history_handler import history
 from awslabs.aws_api_mcp_server.core.aws.driver import translate_cli_to_ir
 from awslabs.aws_api_mcp_server.core.aws.service import (
     execute_awscli_customization,
-    get_local_credentials,
     interpret_command,
     is_operation_read_only,
     validate,
 )
+from awslabs.aws_api_mcp_server.core.common.command import IRCommand
 from awslabs.aws_api_mcp_server.core.common.helpers import as_json
 from awslabs.aws_api_mcp_server.core.common.models import (
     AwsApiMcpServerErrorResponse,
     AwsCliAliasResponse,
     CommandMetadata,
     Context,
-    Credentials,
     InterpretationMetadata,
     InterpretationResponse,
     IRTranslation,
@@ -24,7 +23,6 @@ from awslabs.aws_api_mcp_server.core.common.models import (
 )
 from awslabs.aws_api_mcp_server.core.metadata.read_only_operations_list import ReadOnlyOperations
 from botocore.config import Config
-from botocore.exceptions import NoCredentialsError
 from tests.fixtures import (
     CLOUD9_DESCRIBE_ENVIRONMENTS,
     CLOUD9_LIST_ENVIRONMENTS,
@@ -36,7 +34,6 @@ from tests.fixtures import (
     GET_CALLER_IDENTITY_PAYLOAD,
     SSM_LIST_NODES_PAYLOAD,
     T2_EC2_DESCRIBE_INSTANCES_FILTERED,
-    TEST_CREDENTIALS,
     patch_boto3,
 )
 from typing import Any
@@ -56,10 +53,8 @@ from unittest.mock import MagicMock, patch
 )
 def test_interpret_returns_validation_failures(cli_command, reason, service, operation):
     """Test that interpret_command returns validation failures for invalid operations."""
-    credentials = Credentials(**TEST_CREDENTIALS)
     response = interpret_command(
         cli_command=cli_command,
-        credentials=credentials,
         default_region='us-east-1',
     )
     assert response.response is None
@@ -80,10 +75,8 @@ def test_interpret_returns_validation_failures(cli_command, reason, service, ope
 
 def test_interpret_returns_missing_context_failures():
     """Test that interpret_command returns missing context failures when required parameters are missing."""
-    credentials = Credentials(**TEST_CREDENTIALS)
     response = interpret_command(
         cli_command=CLOUD9_PARAMS_CLI_MISSING_CONTEXT,
-        credentials=credentials,
         default_region='us-east-1',
     )
     assert response.response is None
@@ -165,7 +158,7 @@ def test_interpret_returns_missing_context_failures():
         (
             'aws sts get-caller-identity',
             GET_CALLER_IDENTITY_PAYLOAD,
-            ('GetCallerIdentity', {}, 'us-east-1', 10, 'https://sts.amazonaws.com'),
+            ('GetCallerIdentity', {}, 'us-east-1', 10, 'https://sts.us-east-1.amazonaws.com'),
             'sts',
             'AWS Security Token Service',
             'GetCallerIdentity',
@@ -201,10 +194,7 @@ def test_interpret_returns_valid_response(
     """Test that interpret_command returns a valid response for correct CLI commands."""
     with patch_boto3():
         history.events.clear()
-        credentials = Credentials(**TEST_CREDENTIALS)
-        response = interpret_command(
-            cli_command=cli, default_region='us-east-1', credentials=credentials
-        )
+        response = interpret_command(cli_command=cli, default_region='us-east-1')
         assert response == ProgramInterpretationResponse(
             response=InterpretationResponse(json=as_json(output), error=None, status_code=200),
             failed_constraints=[],
@@ -226,10 +216,8 @@ def test_interpret_injects_region():
         with patch('awslabs.aws_api_mcp_server.core.parser.interpretation.Config') as patch_config:
             history.events.clear()
             patch_config.return_value = default_config
-            credentials = Credentials(**TEST_CREDENTIALS)
             response = interpret_command(
                 cli_command='aws cloud9 describe-environments --environment-ids 7d61007bd98b4d589f1504af84c168de b181ffd35fe2457c8c5ae9d75edc068a',
-                credentials=credentials,
                 default_region=region,
             )
             assert response.metadata == InterpretationMetadata(
@@ -273,11 +261,9 @@ def test_interpret_injects_region():
 def test_region_picked_up_from_arn(cli, region):
     """Test that region is correctly picked up from ARN in the CLI command."""
     with patch_boto3():
-        credentials = Credentials(**TEST_CREDENTIALS)
         response = interpret_command(
             cli_command=cli,
             default_region='us-east-1',
-            credentials=credentials,
         )
         assert response.metadata is not None
         assert response.metadata.region_name == region
@@ -455,67 +441,6 @@ def test_is_operation_read_only_raises_error_for_missing_operation_name():
         is_operation_read_only(ir, read_only_operations)
 
 
-@patch('awslabs.aws_api_mcp_server.core.aws.service.AWS_API_MCP_PROFILE_NAME', 'test')
-@patch('awslabs.aws_api_mcp_server.core.aws.service.boto3.Session')
-def test_get_local_credentials_success_with_aws_mcp_profile(mock_session_class):
-    """Test get_local_credentials returns credentials when available."""
-    mock_session = MagicMock()
-    mock_session_class.return_value = mock_session
-
-    mock_credentials = MagicMock()
-    mock_credentials.access_key = 'test-access-key'
-    mock_credentials.secret_key = 'test-secret-key'  # pragma: allowlist secret
-    mock_credentials.token = 'test-session-token'
-
-    mock_session.get_credentials.return_value = mock_credentials
-
-    result = get_local_credentials()
-
-    assert isinstance(result, Credentials)
-    assert result.access_key_id == 'test-access-key'
-    assert result.secret_access_key == 'test-secret-key'  # pragma: allowlist secret
-    assert result.session_token == 'test-session-token'
-    mock_session_class.assert_called_once_with(profile_name='test')
-    mock_session.get_credentials.assert_called_once()
-
-
-@patch('awslabs.aws_api_mcp_server.core.aws.service.boto3.Session')
-def test_get_local_credentials_success_with_default_creds(mock_session_class):
-    """Test get_local_credentials returns credentials when available."""
-    mock_session = MagicMock()
-    mock_session_class.return_value = mock_session
-
-    mock_credentials = MagicMock()
-    mock_credentials.access_key = 'test-access-key'
-    mock_credentials.secret_key = 'test-secret-key'  # pragma: allowlist secret
-    mock_credentials.token = 'test-session-token'
-
-    mock_session.get_credentials.return_value = mock_credentials
-
-    result = get_local_credentials()
-
-    assert isinstance(result, Credentials)
-    assert result.access_key_id == 'test-access-key'
-    assert result.secret_access_key == 'test-secret-key'  # pragma: allowlist secret
-    assert result.session_token == 'test-session-token'
-    mock_session_class.assert_called_once()
-    mock_session.get_credentials.assert_called_once()
-
-
-@patch('awslabs.aws_api_mcp_server.core.aws.service.boto3.Session')
-def test_get_local_credentials_raises_no_credentials_error(mock_session_class):
-    """Test get_local_credentials raises NoCredentialsError when credentials are None."""
-    mock_session = MagicMock()
-    mock_session_class.return_value = mock_session
-    mock_session.get_credentials.return_value = None
-
-    with pytest.raises(NoCredentialsError):
-        get_local_credentials()
-
-    mock_session_class.assert_called_once()
-    mock_session.get_credentials.assert_called_once()
-
-
 @patch('awslabs.aws_api_mcp_server.core.aws.service.driver')
 def test_execute_awscli_customization_success(mock_driver):
     """Test execute_awscli_customization returns AwsCliAliasResponse on successful execution."""
@@ -528,7 +453,10 @@ def test_execute_awscli_customization_success(mock_driver):
         mock_stderr.getvalue.return_value = ''
         mock_stringio.side_effect = [mock_stdout, mock_stderr]
 
-        result = execute_awscli_customization('aws s3 ls')
+        cli_command = 'aws s3 ls'
+        ir_command = translate_cli_to_ir(cli_command).command
+        assert ir_command is not None
+        result = execute_awscli_customization(cli_command, ir_command)
 
         assert isinstance(result, AwsCliAliasResponse)
         assert result.response == 'bucket1\nbucket2\n'
@@ -542,10 +470,66 @@ def test_execute_awscli_customization_error(mock_driver):
     """Test execute_awscli_customization returns AwsApiMcpServerErrorResponse on exception."""
     mock_driver.main.side_effect = Exception('Invalid command')
 
-    result = execute_awscli_customization('aws invalid command')
+    result = execute_awscli_customization(
+        'aws invalid command',
+        IRCommand(
+            command_metadata=CommandMetadata('invalid', None, 'command'),
+            parameters={},
+            is_awscli_customization=True,
+        ),
+    )
 
     assert isinstance(result, AwsApiMcpServerErrorResponse)
     assert result.error is True
     assert result.detail == "Error while executing 'aws invalid command': Invalid command"
 
     mock_driver.main.assert_called_once_with(['invalid', 'command'])
+
+
+@patch('awslabs.aws_api_mcp_server.core.aws.service.driver.main')
+@patch('awslabs.aws_api_mcp_server.core.aws.service.AWS_API_MCP_PROFILE_NAME', None)
+def test_profile_not_added_when_env_var_none(mock_main):
+    """Test that profile is not added when AWS_API_MCP_PROFILE_NAME is None."""
+    cli_command = 'aws s3 ls'
+    ir_command = translate_cli_to_ir(cli_command).command
+    assert ir_command is not None
+
+    execute_awscli_customization(cli_command, ir_command)
+
+    # Verify profile was not added to args
+    args = mock_main.call_args[0][0]
+    assert '--profile' not in args
+
+
+@patch('awslabs.aws_api_mcp_server.core.aws.service.driver.main')
+@patch('awslabs.aws_api_mcp_server.core.aws.service.AWS_API_MCP_PROFILE_NAME', 'test-profile')
+def test_profile_added_when_env_var_set(mock_main):
+    """Test that profile is added when AWS_API_MCP_PROFILE_NAME is set."""
+    cli_command = 'aws s3 ls'
+    ir_command = translate_cli_to_ir(cli_command).command
+    assert ir_command is not None
+
+    execute_awscli_customization(cli_command, ir_command)
+
+    # Verify profile was added to args
+    args = mock_main.call_args[0][0]
+    assert '--profile' in args
+    profile_index = args.index('--profile')
+    assert args[profile_index + 1] == 'test-profile'
+
+
+@patch('awslabs.aws_api_mcp_server.core.aws.service.driver.main')
+@patch('awslabs.aws_api_mcp_server.core.aws.service.AWS_API_MCP_PROFILE_NAME', 'test-profile')
+def test_profile_not_added_if_present_for_customizations(mock_main):
+    """Test that profile is not added when one is already present."""
+    cli_command = 'aws s3 ls --profile different'
+    ir_command = translate_cli_to_ir(cli_command).command
+    assert ir_command is not None
+
+    execute_awscli_customization(cli_command, ir_command)
+
+    # Verify profile was added to args
+    args = mock_main.call_args[0][0]
+    assert '--profile' in args
+    profile_index = args.index('--profile')
+    assert args[profile_index + 1] == 'different'
