@@ -250,7 +250,7 @@ def test_command_validation_error_for_parameter(command, message):
             + "--sort-criteria 'field=AWS_ACCOUNT_ID,_sortOrder=desc'",
             [
                 "The parameter '--filter-criteria' received an invalid input: "
-                + 'Unknown parameter in input: "myKey", must be one of: awsAccountId,',
+                + 'Unknown parameter in input: "myKey", must be one of: findingArn, awsAccountId,',
                 "\nThe parameter '--sort-criteria' received an invalid input: "
                 + 'Missing required parameter in input: "sortOrder"',
             ],
@@ -630,5 +630,10 @@ def test_valid_expand_user_home_directory():
 def test_invalid_expand_user_home_directory():
     """Test that tilde is not replaced."""
     result = parse(cli_command='aws s3 cp s3://my_file ~user_that_does_not_exist/temp/test.txt')
-    print(result)
     assert any(param.startswith('~') for param in result.parameters['--paths'])
+
+
+def test_profile():
+    """Test that the profile is correctly extracted."""
+    result = parse(cli_command='aws s3api list-buckets --profile test-profile')
+    assert result.profile == 'test-profile'
