@@ -3,17 +3,18 @@ from awslabs.aws_api_mcp_server.core.common.errors import AwsApiMcpError
 from awslabs.aws_api_mcp_server.core.common.models import (
     AwsApiMcpServerErrorResponse,
     AwsCliAliasResponse,
+    Consent,
     InterpretationResponse,
     ProgramInterpretationResponse,
 )
 from awslabs.aws_api_mcp_server.server import call_aws, main, suggest_aws_commands
 from botocore.exceptions import NoCredentialsError
-from tests.fixtures import TEST_CREDENTIALS, DummyCtx
+from mcp.server.elicitation import AcceptedElicitation
+from tests.fixtures import DummyCtx
 from unittest.mock import AsyncMock, MagicMock, patch
 
 
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
 @patch('awslabs.aws_api_mcp_server.server.interpret_command')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
@@ -23,11 +24,8 @@ async def test_call_aws_success(
     mock_translate_cli_to_ir,
     mock_validate,
     mock_interpret,
-    mock_get_creds,
 ):
     """Test call_aws returns success for a valid read-only command."""
-    mock_get_creds.return_value = TEST_CREDENTIALS
-
     # Create a proper ProgramInterpretationResponse mock
     mock_response = InterpretationResponse(error=None, json='{"Buckets": []}', status_code=200)
 
@@ -62,7 +60,6 @@ async def test_call_aws_success(
     assert result == mock_result
     mock_translate_cli_to_ir.assert_called_once_with('aws s3api list-buckets')
     mock_validate.assert_called_once_with(mock_ir)
-    mock_get_creds.assert_called_once()
     mock_interpret.assert_called_once()
 
 
@@ -114,21 +111,115 @@ async def test_suggest_aws_commands_exception(mock_knowledge_base):
 
 
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
+@patch('awslabs.aws_api_mcp_server.server.REQUIRE_MUTATION_CONSENT', True)
 @patch('awslabs.aws_api_mcp_server.server.interpret_command')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
 @patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
-async def test_call_aws_with_mutating_action(
+async def test_call_aws_with_consent_and_accept(
     mock_is_operation_read_only,
     mock_translate_cli_to_ir,
     mock_validate,
     mock_interpret,
-    mock_get_creds,
 ):
-    """Test call_aws with mutating action."""
-    mock_get_creds.return_value = TEST_CREDENTIALS
+    """Test call_aws with mutating action and consent enabled."""
+    # Create a proper ProgramInterpretationResponse mock
+    mock_response = InterpretationResponse(error=None, json='{"Buckets": []}', status_code=200)
 
+    mock_result = ProgramInterpretationResponse(
+        response=mock_response,
+        metadata=None,
+        validation_failures=None,
+        missing_context_failures=None,
+        failed_constraints=None,
+    )
+    mock_interpret.return_value = mock_result
+
+    mock_is_operation_read_only.return_value = False
+
+    # Mock IR with command metadata
+    mock_ir = MagicMock()
+    mock_ir.command_metadata = MagicMock()
+    mock_ir.command_metadata.service_sdk_name = 's3api'
+    mock_ir.command_metadata.operation_sdk_name = 'create-bucket'
+    mock_ir.command = MagicMock()
+    mock_ir.command.is_awscli_customization = False  # Ensure interpret_command is called
+    mock_translate_cli_to_ir.return_value = mock_ir
+
+    mock_response = MagicMock()
+    mock_response.validation_failed = False
+    mock_validate.return_value = mock_response
+
+    mock_ctx = AsyncMock()
+    mock_ctx.elicit.return_value = AcceptedElicitation(data=Consent(answer=True))
+
+    # Execute
+    result = await call_aws('aws s3api create-bucket --bucket somebucket', mock_ctx)
+
+    # Verify that consent was requested
+    assert result == mock_result
+    mock_translate_cli_to_ir.assert_called_once_with('aws s3api create-bucket --bucket somebucket')
+    mock_validate.assert_called_once_with(mock_ir)
+    mock_interpret.assert_called_once()
+    mock_ctx.elicit.assert_called_once()
+
+
+@patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
+@patch('awslabs.aws_api_mcp_server.server.REQUIRE_MUTATION_CONSENT', True)
+@patch('awslabs.aws_api_mcp_server.server.interpret_command')
+@patch('awslabs.aws_api_mcp_server.server.validate')
+@patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
+@patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
+async def test_call_aws_with_consent_and_reject(
+    mock_is_operation_read_only,
+    mock_translate_cli_to_ir,
+    mock_validate,
+    mock_interpret,
+):
+    """Test call_aws with mutating action and consent enabled."""
+    mock_response = InterpretationResponse(error=None, json='{"Buckets": []}', status_code=200)
+    mock_is_operation_read_only.return_value = False
+
+    # Mock IR with command metadata
+    mock_ir = MagicMock()
+    mock_ir.command_metadata = MagicMock()
+    mock_ir.command_metadata.service_sdk_name = 's3api'
+    mock_ir.command_metadata.operation_sdk_name = 'create-bucket'
+    mock_ir.command = MagicMock()
+    mock_ir.command.is_awscli_customization = False
+    mock_translate_cli_to_ir.return_value = mock_ir
+
+    mock_response = MagicMock()
+    mock_response.validation_failed = False
+    mock_validate.return_value = mock_response
+
+    mock_ctx = AsyncMock()
+    mock_ctx.elicit.return_value = AcceptedElicitation(data=Consent(answer=False))
+
+    # Execute
+    result = await call_aws('aws s3api create-bucket --bucket somebucket', mock_ctx)
+
+    # Verify that consent was requested
+    assert result == AwsApiMcpServerErrorResponse(
+        detail='Error while executing the command: User rejected the execution of the command.'
+    )
+    mock_translate_cli_to_ir.assert_called_once_with('aws s3api create-bucket --bucket somebucket')
+    mock_validate.assert_called_once_with(mock_ir)
+
+
+@patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
+@patch('awslabs.aws_api_mcp_server.server.REQUIRE_MUTATION_CONSENT', False)
+@patch('awslabs.aws_api_mcp_server.server.interpret_command')
+@patch('awslabs.aws_api_mcp_server.server.validate')
+@patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
+@patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
+async def test_call_aws_without_consent(
+    mock_is_operation_read_only,
+    mock_translate_cli_to_ir,
+    mock_validate,
+    mock_interpret,
+):
+    """Test call_aws with mutating action and with consent disabled."""
     # Create a proper ProgramInterpretationResponse mock
     mock_response = InterpretationResponse(error=None, json='{"Buckets": []}', status_code=200)
 
@@ -159,11 +250,10 @@ async def test_call_aws_with_mutating_action(
     # Execute
     result = await call_aws('aws s3api create-bucket --bucket somebucket', DummyCtx())
 
-    # Verify that no consent was requested
+    # Verify that consent was requested
     assert result == mock_result
     mock_translate_cli_to_ir.assert_called_once_with('aws s3api create-bucket --bucket somebucket')
     mock_validate.assert_called_once_with(mock_ir)
-    mock_get_creds.assert_called_once()
     mock_interpret.assert_called_once()
 
 
@@ -200,12 +290,12 @@ async def test_call_aws_validation_error_generic_exception(mock_translate_cli_to
     )
 
 
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
+@patch('awslabs.aws_api_mcp_server.server.interpret_command', side_effect=NoCredentialsError())
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
 @patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
 async def test_call_aws_no_credentials_error(
-    mock_is_operation_read_only, mock_translate_cli_to_ir, mock_validate, mock_get_creds
+    mock_is_operation_read_only, mock_translate_cli_to_ir, mock_validate, mock_interpret
 ):
     """Test call_aws returns error when no AWS credentials are found."""
     # Mock IR with command metadata
@@ -224,8 +314,6 @@ async def test_call_aws_no_credentials_error(
     mock_response.validation_failed = False
     mock_validate.return_value = mock_response
 
-    mock_get_creds.side_effect = NoCredentialsError()
-
     # Execute
     result = await call_aws('aws s3api list-buckets', DummyCtx())
 
@@ -238,7 +326,6 @@ async def test_call_aws_no_credentials_error(
 
 
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
 @patch('awslabs.aws_api_mcp_server.server.interpret_command')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
@@ -248,11 +335,8 @@ async def test_call_aws_execution_error_awsmcp_error(
     mock_translate_cli_to_ir,
     mock_validate,
     mock_interpret,
-    mock_get_creds,
 ):
     """Test call_aws returns error details for AwsApiMcpError during execution."""
-    mock_get_creds.return_value = TEST_CREDENTIALS
-
     # Mock IR with command metadata
     mock_ir = MagicMock()
     mock_ir.command_metadata = MagicMock()
@@ -285,7 +369,6 @@ async def test_call_aws_execution_error_awsmcp_error(
 
 
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
 @patch('awslabs.aws_api_mcp_server.server.interpret_command')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
@@ -295,11 +378,8 @@ async def test_call_aws_execution_error_generic_exception(
     mock_translate_cli_to_ir,
     mock_validate,
     mock_interpret,
-    mock_get_creds,
 ):
     """Test call_aws returns error details for generic exception during execution."""
-    mock_get_creds.return_value = TEST_CREDENTIALS
-
     # Mock IR with command metadata
     mock_ir = MagicMock()
     mock_ir.command_metadata = MagicMock()
@@ -479,7 +559,6 @@ async def test_call_aws_both_validation_failures_and_constraints(
 
 
 @patch('awslabs.aws_api_mcp_server.server.execute_awscli_customization')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
 @patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
@@ -487,7 +566,6 @@ async def test_call_aws_awscli_customization_success(
     mock_is_operation_read_only,
     mock_translate_cli_to_ir,
     mock_validate,
-    mock_get_creds,
     mock_execute_awscli_customization,
 ):
     """Test call_aws returns success response for AWS CLI customization command."""
@@ -497,7 +575,6 @@ async def test_call_aws_awscli_customization_success(
     mock_translate_cli_to_ir.return_value = mock_ir
 
     mock_is_operation_read_only.return_value = True
-    mock_get_creds.return_value = TEST_CREDENTIALS
 
     mock_response = MagicMock()
     mock_response.validation_failed = False
@@ -511,12 +588,12 @@ async def test_call_aws_awscli_customization_success(
     assert result == expected_response
     mock_translate_cli_to_ir.assert_called_once_with('aws configure list')
     mock_validate.assert_called_once_with(mock_ir)
-    mock_execute_awscli_customization.assert_called_once_with('aws configure list')
-    mock_get_creds.assert_called_once()
+    mock_execute_awscli_customization.assert_called_once_with(
+        'aws configure list', mock_ir.command
+    )
 
 
 @patch('awslabs.aws_api_mcp_server.server.execute_awscli_customization')
-@patch('awslabs.aws_api_mcp_server.server.get_local_credentials')
 @patch('awslabs.aws_api_mcp_server.server.validate')
 @patch('awslabs.aws_api_mcp_server.server.translate_cli_to_ir')
 @patch('awslabs.aws_api_mcp_server.server.is_operation_read_only')
@@ -524,7 +601,6 @@ async def test_call_aws_awscli_customization_error(
     mock_is_operation_read_only,
     mock_translate_cli_to_ir,
     mock_validate,
-    mock_get_creds,
     mock_execute_awscli_customization,
 ):
     """Test call_aws handles error response from AWS CLI customization command."""
@@ -534,7 +610,6 @@ async def test_call_aws_awscli_customization_error(
     mock_translate_cli_to_ir.return_value = mock_ir
 
     mock_is_operation_read_only.return_value = True
-    mock_get_creds.return_value = TEST_CREDENTIALS
 
     mock_response = MagicMock()
     mock_response.validation_failed = False
@@ -553,33 +628,25 @@ async def test_call_aws_awscli_customization_error(
     assert result == error_response
     mock_translate_cli_to_ir.assert_called_once_with('aws configure list')
     mock_validate.assert_called_once_with(mock_ir)
-    mock_execute_awscli_customization.assert_called_once_with('aws configure list')
+    mock_execute_awscli_customization.assert_called_once_with(
+        'aws configure list', mock_ir.command
+    )
     mock_ctx.error.assert_called_once_with(error_response.detail)
-    mock_get_creds.assert_called_once()
 
 
+@patch('awslabs.aws_api_mcp_server.core.kb.threading.Thread')
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', None)
 @patch('awslabs.aws_api_mcp_server.server.WORKING_DIRECTORY', '/tmp')
-def test_main_missing_aws_region():
+def test_main_missing_aws_region(mock_thread):
     """Test main function raises ValueError when AWS_REGION environment variable is not set."""
     with pytest.raises(ValueError, match=r'AWS_REGION environment variable is not defined.'):
         main()
 
 
-@patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
-@patch('awslabs.aws_api_mcp_server.server.WORKING_DIRECTORY', None)
-def test_main_missing_working_directory():
-    """Test main function raises ValueError when AWS_API_MCP_WORKING_DIR environment variable is not set."""
-    with pytest.raises(
-        ValueError,
-        match=r'AWS_API_MCP_WORKING_DIR environment variable is not defined.',
-    ):
-        main()
-
-
+@patch('awslabs.aws_api_mcp_server.core.kb.threading.Thread')
 @patch('awslabs.aws_api_mcp_server.server.DEFAULT_REGION', 'us-east-1')
 @patch('awslabs.aws_api_mcp_server.server.WORKING_DIRECTORY', 'relative/path')
-def test_main_relative_working_directory():
+def test_main_relative_working_directory(mock_thread):
     """Test main function raises ValueError when AWS_API_MCP_WORKING_DIR is a relative path."""
     with pytest.raises(
         ValueError,
@@ -588,6 +655,7 @@ def test_main_relative_working_directory():
         main()
 
 
+@patch('awslabs.aws_api_mcp_server.core.kb.threading.Thread')
 @patch('awslabs.aws_api_mcp_server.server.os.chdir')
 @patch('awslabs.aws_api_mcp_server.server.server')
 @patch('awslabs.aws_api_mcp_server.server.get_read_only_operations')
@@ -600,6 +668,7 @@ def test_main_success_with_read_only_mode(
     mock_get_read_only_operations,
     mock_server,
     mock_chdir,
+    mock_thread,
 ):
     """Test main function executes successfully with read-only mode enabled."""
     mock_knowledge_base.setup = MagicMock()
